#include <stdrsl/ShadingContext.h>
#include <stdrsl/RadianceSample.h>
#include <stdrsl/Math.h>
#include <stdrsl/Colors.h>
#include <stdrsl/OrenNayar.h>
#include <stdrsl/SpecularDBRDF.h>
#include <stdrsl/Fresnel.h>

//A simple class-based shader
class dielectricShader(
    //Diffuse
    uniform float roughness = 0.2;
    uniform color diffuse_color = color(1.0);
    uniform string diffuse_map = "";
    uniform float diffuse_intensity = 0.5;
    //Specular
    uniform float gloss = 0.001;
    uniform color specular_color = color(1.0);
    uniform string specular_map = "";
    uniform float specular_intensity = 1.0;
    //Fresnel
    uniform float fesnel_amount = 1.0;
    uniform float ior = 1.33;
    uniform float media_ior = 1;
    //Other
    uniform float gamma_correct = 1;
    )
{
  //Class members
  shader m_lights[];
  shader m_material;
  stdrsl_ShadingContext m_context;
  stdrsl_OrenNayar m_diffuse;
  stdrsl_SpecularDBRDF m_specular;
  stdrsl_Fresnel m_fresnel;

  varying color diffuse;
  varying color specular;
  
  //Pipeline methods
  public void construct()
  {
    m_context->construct();
  }

  public void begin()
  {
    m_lights = getlights();
    m_material = this;
    m_context->init();
  }

  public void displacement(output point P; output normal N)
  {
    normal diff = normalize(N) - normalize(Ng);

    varying point Pw = transform("object", m_context->m_Ps);
    Pw = point(Pw[0] * 1.0, Pw[1] * 3.0, Pw[2] * 1.0);

    varying float noise_one = (float noise(Pw) - 0.5);
    varying float noise_two = (float noise(Pw * 2) - 0.5) / 2;
    varying float noise_three = (float noise(Pw * 4) - 0.5) / 4;
    varying float fbm = abs(noise_one + noise_two + noise_three) * 0.025;

    varying point Pt = m_context->m_Ps + m_context->m_Nn * fbm;
    varying normal Nf = normalize(calculatenormal(Pt));

    m_context->m_Nn = Nf + diff;
    m_context->reinit();
  }

  public void opacity(output color Oi;)
  {
    //No opacity option implememnted
  }

  //Lighting methods
  public void prelighting(output color Ci, Oi;)
  {
    //Initialize fresnel values
    m_fresnel->init(m_context, media_ior, ior);

    //Account for pigment variation
    varying point Pw = transform("object", m_context->m_Ps);
    varying float noise = (float noise(Pw * 20) - 0.5);
    varying float smoothstp = smoothstep(0.20, 0.24, noise);

    //Get diffuse colour and initialize diffuse brdf
    diffuse = diffuse_color;
    if(diffuse_map != "")
    {
      diffuse = color texture(diffuse_map);
      if(gamma_correct == 1)
        diffuse = srgbToLinear(diffuse);
    }

    m_diffuse->init(diffuse * diffuse_intensity * max(1 - smoothstp, 0.75), roughness);

    //Get specular colour and initialize specular brdf
    specular = specular_color;
    if(specular_map != "")
    {
      specular = color texture(specular_map);
      if(gamma_correct == 1)
        specular = srgbToLinear(specular);
    }

    m_specular->init(m_context, "beckmann", specular * specular_intensity, gloss, 16, 32);
  }

  public void lighting(output color Ci, Oi;)
  {
    __radiancesample samples[];

    //Needs to come before directlighting() to populate samples from material before light
    color indirect_spec = indirectspecular(this, "materialsamples", samples);

    //Calculate direct lighting using material/light methods and multiple importance sampling 
    color direct_diff;
    color direct_spec;
    directlighting(
      m_material,
      m_lights,
      "mis", 1,
      "heuristic", "veachpower2",
      "diffuseresult", direct_diff,
      "specularresult", direct_spec,
      "materialsamples", samples
      );

    //Calculate sample counts based on ray depth and path tracing attribute
    uniform float idiffSamps, idiffMaxVar;
    m_context->m_SampleMgr->computeIndirectDiffuseSamples(64, 0, idiffSamps, idiffMaxVar);

    //Jitter the surface position to avoid aliasing artifacts on secondary rays
    float sample_base = (m_context->m_RayDepth > 0) ? 0 : 1;

    //Get indirect diffuse contribution using cosine weighted distrobution to account for projected area
    color indirect_diff = indirectdiffuse(
      P,
      m_context->m_Nn,
      idiffSamps, 
      "distribution", "cosine",
      "samplebase", sample_base,
      "maxvariation", idiffMaxVar
      ) * m_diffuse->m_diffColor;

    //Accumilate individual contributions
    Ci = (direct_diff + indirect_diff) * m_fresnel->m_Kt 
    + (direct_spec + indirect_spec) * m_fresnel->m_Kr;
  }

  //Sample evaluate and generate methods
  public void evaluateSamples(string distribution; output __radiancesample samples[])
  {
    if(distribution == "diffuse")
    {
      m_diffuse->evalDiffuseSamps(m_context, m_fresnel, samples);
    }
    else
    {
      m_specular->evalSpecularSamps(m_context, m_fresnel, samples);
    }
  }

  public void generateSamples(string distribution; output __radiancesample samples[])
  {
    if(distribution == "diffuse")
    {
      m_diffuse->genDiffuseSamps(m_context, m_fresnel, samples);
    }
    else
    {
      m_specular->genSpecularSamps(m_context, m_fresnel, "", samples);
    }
  }
}